#!/usr/bin/env python
#

from gimpfu import *
import decimal
import random
import time
import datetime

############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    gimp.message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
############################################################################
#

def ChildishEngrave (img, layer, howmany, patfilter) :
    
    pdb.gimp_context_push()

    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(img)
    
    sourcename = layer.name
    if '.' in sourcename:
        split = sourcename.split(".", 1)
        sourcename = split[0]
        layer.name = sourcename
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime('%Y-%m-%d %H-%M-%S')
    
    # simplify
    pdb.plug_in_sel_gauss (img, layer, 16.0, 48)
    
    # Get the layer position.
    pos = 0;
    for i in range(len(img.layers)):
        if(img.layers[i] == layer):
            pos = i
            
    # Create a new layer to save the results (otherwise is not possible to undo the operation).
    newLayer = layer.copy()
    newLayer.name = layer.name + " copy (newLayer)"

    img.add_layer(newLayer, pos)
    
    # change image mode to indexed  

    pdb.gimp_image_convert_indexed (img, 0, 0, howmany, 0, 0, "ignored")
    is_indexed=pdb.gimp_drawable_is_indexed (newLayer) 
    if is_indexed==1:
        hidden_message="OK, converted"
    else:    
        erro_msg ("Image not converted to indexed, exit")
        pdb.gimp_context_pop
        return    

    # get the number of pixels in the colormap (which exists for indexed images)
    # ==========================================================================
    (count, colors)=pdb.gimp_image_get_colormap(img)
    # the count is nr_of_colors * 3, example 8*3=24
    nr_areas=(count/3) 
    #info_msg("nr of colors in the palette: "+str(nr_areas)) 
  
    temp_palette=pdb.gimp_palette_new(pdb.gimp_image_get_name(img))
    
    index=0
    while index<count:
    # Compute the color
        color= (colors[index], colors[index+1], colors[index+2])
        #info_msg ("color = "+str(color))
    # Add to the filter temporary palette
        pdb.gimp_palette_add_entry(temp_palette, str(index/3), color)
        index+=3 
    # Endwhile index<count
  
    # the temporary palette has been created            
    # ======================================

    # re-set image to RGB to proceed
    # ==============================
    pdb.gimp_image_convert_rgb (img)
    
    bordersLayer = newLayer.copy()
    bordersLayer.name = layer.name + " borders to engrave (bordersLayer)"

    img.add_layer(bordersLayer, pos+1)  # was pos
    
    (mean, stddev, median, pixels, count, percent) = pdb.gimp_histogram (newLayer, 0, 0, 255)
    avgpixels = pixels / nr_areas   # average number of pixels per colour area
    bigareapixels = avgpixels * 3   # define only a few areas as big, likely just one
    midareapixels = avgpixels       # define some medium colour areas
    smlareapixels = avgpixels / 9   # define some small colour areas
    
    bu = max(2,(img.width+img.height+500)/1000)
     
    pdb.gimp_context_set_foreground ((0,0,0,255))
    areaNR=0
    area_color=[]   # empty list
    for areaNR in range(nr_areas):
        one_color=pdb.gimp_palette_entry_get_color(temp_palette, areaNR)
        pdb.gimp_image_select_color(img, CHANNEL_OP_REPLACE, newLayer, one_color)
        (there_is_a_selection, x1, y1, x2, y2) = pdb.gimp_selection_bounds(img)
        if there_is_a_selection==1:
            (mean, stddev, median, pixels, count, percent) = pdb.gimp_histogram (newLayer, 0, 0, 255)
            if pixels >= bigareapixels:
                    border = bu*2
            elif pixels >= midareapixels:
                    border = bu 
            else:
                    border = 1  # fixed for small areas
            pdb.gimp_selection_border (img, border)
            pdb.gimp_edit_fill (bordersLayer, FOREGROUND_FILL)
                
        else:
            erro_msg ("no selection for colour ?"+str(one_color))

    ##info_msg ("end loop")
    pdb.gimp_selection_none (img)

    pdb.gimp_threshold (bordersLayer, 2, 255)
    pdb.plug_in_colortoalpha (img, bordersLayer, ((255,255,255,255)))

    (np, LPATT) = pdb.gimp_patterns_get_list (patfilter)       
    if np>0:
        rn = random.randrange (np)
        pdb.gimp_context_set_pattern (LPATT[rn])
        #info_msg (str(LPATT[rn]))
        pdb.gimp_edit_fill (newLayer, PATTERN_FILL)
        
    pdb.plug_in_spread (img, bordersLayer, 2, 2)
    newLayer.name = layer.name + " pattern="+LPATT[rn]+" (newLayer)"
    
    newLayer.mode = GRAIN_MERGE_MODE
    newLayer.opacity = 80.0
    
    # more saturation
    pdb.gimp_hue_saturation (layer, 0, 0, 0, +50)
    
    finalLayer = pdb.gimp_layer_new_from_visible (img, img, st+" "+sourcename+" childish engraved (finalLayer)")
    img.add_layer(finalLayer, pos)
    
    # Close the undo group.
    pdb.gimp_image_undo_group_end(img)
    
    # End process.
    pdb.gimp_context_pop()
    
    return(finalLayer)

register(
    "ChildishEngrave",    "ChildishEngrave",    "ChildishEngrave.",
    "Dinasset",    "Open source (BSD 3-clause license)",    "2017",
    "ChildishEngrave",
    "RGB, RGB*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_ADJUSTMENT, "colours", "How many colours",  9, (4, 255, 1)), 		#default, (min, max, increment)
      (PF_STRING, "filter", "Filter for wood patterns names"+
                             " \n (if none is found engrave on image):", "wood of some sort"),
    ],
    [
      (PF_DRAWABLE, "outdrawable", "Output drawable", None),
    ],
    ChildishEngrave,
    menu="<Image>/Diego/Ancillary"
    )

main()
